@@ -5,26 +5,25 @@ use 5.005;
 use strict;
 use vars qw($VERSION);
 
-use fields qw( index frames );
+use File::Spec;
 
 use overload
     '""' => \&as_string,
     fallback => 1;
 
-$VERSION = '1.04';
-
-1;
+$VERSION = '1.12';
 
 sub new
 {
-    my $proto = shift;
-    my $class = ref $proto || $proto;
+    my $class = shift;
+    my %p = @_;
 
     my $self = bless { index => undef,
 		       frames => [],
+                       respect_overload => $p{respect_overload},
 		     }, $class;
 
-    $self->_add_frames(@_);
+    $self->_add_frames(%p);
 
     return $self;
 }
@@ -36,11 +35,13 @@ sub _add_frames
 
     $p{no_refs} = delete $p{no_object_refs} if exists $p{no_object_refs};
 
-    my (%i_pack, %i_class);
+    my (@i_pack_re, %i_class);
     if ($p{ignore_package})
     {
-	$p{ignore_package} = [$p{ignore_package}] unless ref $p{ignore_package};
-	%i_pack = map {$_ => 1} @{ $p{ignore_package} };
+	$p{ignore_package} =
+            [$p{ignore_package}] unless UNIVERSAL::isa( $p{ignore_package}, 'ARRAY' );
+
+        @i_pack_re = map { ref $_ ? $_ : qr/^\Q$_\E$/ } @{ $p{ignore_package} };
     }
 
     if ($p{ignore_class})
@@ -50,14 +51,13 @@ sub _add_frames
     }
 
     my $p = __PACKAGE__;
-    $i_pack{$p} = 1;
+    push @i_pack_re, qr/^\Q$p\E$/;
 
     my $x = 0;
     my @c;
-    while ( do { package DB; @c = caller($x++) } )
+    while ( do { package DB; @DB::args = (); @c = caller($x++) } )
     {
-	# Do the quickest ones first.
-	next if $i_pack{ $c[0] };
+        next if grep { $c[0] =~ /$_/ } @i_pack_re;
 	next if grep { $c[0]->isa($_) } keys %i_class;
 
         $self->_add_frame( $p{no_refs}, \@c )
@@ -94,13 +94,33 @@ sub _add_frame
                                   # versions of E::C::B
                                   $_->{message};
                               } }
-                         : "$_"
+                         : $self->_ref_as_string($_)
                        )
                      : $_
                    ) } @a;
     }
 
-    push @{ $self->{frames} }, Devel::StackTraceFrame->new( $c, \@a );
+    push @{ $self->{frames} },
+        Devel::StackTraceFrame->new( $c, \@a, $self->{respect_overload} );
+}
+
+sub _ref_as_string
+{
+    my $self = shift;
+
+    local $@;
+    if ( ref $_[0] &&
+         ! $self->{respect_overload} &&
+         eval { overload::Overloaded($_[0]) }
+       )
+    {
+        return overload::StrVal($_[0]);
+    }
+    else
+    {
+        # force stringification
+        $_[0] . '';
+    }
 }
 
 sub next_frame
@@ -190,8 +210,6 @@ package Devel::StackTraceFrame;
 use strict;
 use vars qw($VERSION);
 
-use fields qw( package filename line subroutine hasargs wantarray evaltext is_require hints bitmask args );
-
 $VERSION = '0.6';
 
 # Create accessor routines
@@ -219,7 +237,12 @@ sub new
 
     @{ $self }{ @fields } = @{$_[0]};
 
-    $self->{args} = $_[1] ? $_[1] : [];
+    # fixup unix-style paths on win32
+    $self->{filename} = File::Spec->canonpath( $self->{filename} );
+
+    $self->{args} = $_[1];
+
+    $self->{respect_overload} = $_[2];
 
     return $self;
 }
@@ -277,8 +300,9 @@ sub as_string
 		# set args to the string "undef" if undefined
 		$_ = "undef", next unless defined $_;
 
-		# force stringification
-		$_ .= '' if ref $_;
+                # hack!
+                $_ = $self->Devel::StackTrace::_ref_as_string($_)
+                    if ref $_;
 
 		s/'/\\'/g;
 
@@ -373,12 +397,14 @@ Returns a new Devel::StackTrace object.
 
 Takes the following parameters:
 
-=item -- ignore_package => $package_name OR \@package_names
+=over 8
+
+=item * ignore_package => $package_name OR \@package_names
 
 Any frames where the package is one of these packages will not be on
 the stack.
 
-=item -- ignore_class => $package_name OR \@package_names
+=item * ignore_class => $package_name OR \@package_names
 
 Any frames where the package is a subclass of one of these packages
 (or is the same package) will not be on the stack.
@@ -388,7 +414,7 @@ parameter, meaning that the Devel::StackTrace package is B<ALWAYS>
 ignored.  However, if you create a subclass of Devel::StackTrace it
 will not be ignored.
 
-=item -- no_refs => $boolean
+=item * no_refs => $boolean
 
 If this parameter is true, then Devel::StackTrace will not store
 references internally when generating stacktrace frames.  This lets
@@ -397,6 +423,16 @@ your objects go out of scope.
 Devel::StackTrace replaces any references with their stringified
 representation.
 
+=item * respect_overload => $boolean
+
+By default, Devel::StackTrace will call C<overload::StrVal()> to get
+the underlying string representation of an object, instead of
+respecting the object's stringification overloading.  If you would
+prefer to see the overloaded representation of objects in stack
+traces, then set this parameter to true.
+
+=back
+
 =item * next_frame
 
 Returns the next Devel::StackTraceFrame object down on the stack.  If
@@ -473,12 +509,18 @@ Returns undef if the frame was not part of a require.
 Returns the arguments passed to the frame.  Note that any arguments
 that are references are returned as references, not copies.
 
+=back
+
 =head2 These only contain data as of Perl 5.6.0 or later
 
+=over 4
+
 =item * hints
 
 =item * bitmask
 
+=back
+
 =head1 AUTHOR
 
 Dave Rolsky, <autarch@urth.org>
